﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Ozeki.Media;
using Ozeki.Media.MediaHandlers;
using Ozeki.Network.Nat;
using Ozeki.VoIP;
using Ozeki.VoIP.Media;
using Ozeki.VoIP.SDK;
using Ozeki.Common;


namespace AutoAnsweringSoftphone
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        public bool IsRegRequired = false;
        public String displayname;
        public String username;
        public String registername;
        public String regpass;
        public String domainhost;
        public int port;
        public NatTraversalMethod natTraversal = 0;

        ISoftPhone softPhone;
        IPhoneLine phoneLine;
        PhoneLineState phoneLineInformation;
        IPhoneCall call;
        Microphone microphone = Microphone.GetDefaultDevice();
        Speaker speaker = Speaker.GetDefaultDevice();
        MediaConnector connector = new MediaConnector();
        PhoneCallAudioSender mediaSender = new PhoneCallAudioSender();
        PhoneCallAudioReceiver mediaReceiver = new PhoneCallAudioReceiver();
        bool inComingCall;

        private void InitializeSoftPhone()
        {
            try
            {
                softPhone = SoftPhoneFactory.CreateSoftPhone(SoftPhoneFactory.GetLocalIP(), 5700, 5750, 5700);
                softPhone.IncomingCall += new EventHandler<VoIPEventArgs<IPhoneCall>>(softPhone_IncomingCall);

                SIPAccount sa = new SIPAccount(IsRegRequired, displayname, username, registername, regpass, domainhost, port);
                NatConfiguration nc = new NatConfiguration(natTraversal);

                phoneLine = softPhone.CreatePhoneLine(sa, nc);
                phoneLine.PhoneLineStateChanged += new EventHandler<VoIPEventArgs<PhoneLineState>>(phoneLine_PhoneLineInformation);

                softPhone.RegisterPhoneLine(phoneLine);
            }
            catch (Exception ex)
            {
                MessageBox.Show(String.Format("You didn't set the right IP address. Please set your IP address and try again\n {0}", ex.Message), string.Empty, MessageBoxButtons.OK,
                MessageBoxIcon.Error);
            }
        }

        private void InvokeGUIThread(Action action)
        {
            Invoke(action);
        }

        private void softPhone_IncomingCall(object sender, VoIPEventArgs<IPhoneCall> e)
        {
            InvokeGUIThread(() =>
            {
                RegistrationState.Text = "Incoming call";
                DialedNumber.Text = String.Format("from {0}\n", e.Item.DialInfo);
                call = e.Item;
                WireUpCallEvents();
                inComingCall = true;
                call.Accept();
            });
        }

        private void WireUpCallEvents()
        {
            call.CallStateChanged += new EventHandler<VoIPEventArgs<CallState>>(call_CallStateChanged);
            call.CallErrorOccured += new EventHandler<VoIPEventArgs<CallError>>(call_CallErrorOccured);
        }

        private void WireDownCallEvents()
        {
            if (call != null)
            {
                call.CallStateChanged -= (call_CallStateChanged); ;
                call.CallErrorOccured -= (call_CallErrorOccured);
            }
        }

        private void call_CallErrorOccured(object sender, VoIPEventArgs<CallError> e)
        {
            InvokeGUIThread(() =>
            {
                RegistrationState.Text = e.Item.ToString();
            });
        }

        private void call_CallStateChanged(object sender, VoIPEventArgs<CallState> e)
        {
            InvokeGUIThread(() => { RegistrationState.Text = e.Item.ToString(); });

            switch (e.Item)
            {
                case CallState.InCall:
                    microphone.Start();
                    connector.Connect(microphone, mediaSender);

                    speaker.Start();
                    connector.Connect(mediaReceiver, speaker);

                    mediaSender.AttachToCall(call);
                    mediaReceiver.AttachToCall(call);

                    break;
                case CallState.Completed:
                    microphone.Stop();
                    connector.Disconnect(microphone, mediaSender);

                    speaker.Stop();
                    connector.Disconnect(mediaReceiver, speaker);

                    mediaSender.Detach();
                    mediaReceiver.Detach();

                    WireDownCallEvents();
                    call = null;

                    InvokeGUIThread(() => { DialedNumber.Text = string.Empty; });
                    break;
                case CallState.Cancelled:
                    WireDownCallEvents();
                    call = null;
                    break;
            }
        }

        private void phoneLine_PhoneLineInformation(object sender, VoIPEventArgs<PhoneLineState> e)
        {
            phoneLineInformation = e.Item;
            InvokeGUIThread(() =>
            {
                UsernameL.Text = ((IPhoneLine)sender).SIPAccount.RegisterName;
                if (e.Item == PhoneLineState.RegistrationSucceeded)
                {
                    ConnectionState.Text = "Online";
                    RegistrationState.Text = "Registration succeeded";
                }
                else
                    RegistrationState.Text = e.Item.ToString();
            });
        }

        private void OKButton_Click(object sender, EventArgs e)
        {
            displayname = DisplayNameTB.Text;
            username = UserNameTB.Text;
            registername = RegNameTB.Text;
            regpass = RegPwdTB.Text;

            try
            {
                int i;
                i = Int32.Parse(IP1.Text);
                if (!(i >= 0 && i <= 255))
                    throw new Exception();
                i = Int32.Parse(IP2.Text);
                if (!(i >= 0 && i <= 255))
                    throw new Exception();
                i = Int32.Parse(IP3.Text);
                if (!(i >= 0 && i <= 255))
                    throw new Exception();
                i = Int32.Parse(IP4.Text);
                if (!(i >= 0 && i <= 255))
                    throw new Exception();
            }
            catch (Exception ex)
            {
                MessageBox.Show(String.Format("The IP address you set is invalid, please set a valid IP address.\n {0}", ex.Message), string.Empty, MessageBoxButtons.OK,
            MessageBoxIcon.Error);
            }
            domainhost = string.Empty;
            domainhost = IP1.Text + "." + IP2.Text + "." + IP3.Text + "." + IP4.Text;
            try
            {
                port = Int32.Parse(PortNo.Text);
            }
            catch (Exception ex)
            {

                MessageBox.Show(String.Format("The port number you set is invalid, please set a valid port number.\n {0}", ex.Message), string.Empty, MessageBoxButtons.OK,
            MessageBoxIcon.Error);
            }
            switch (NAT.SelectedIndex)
            {
                case -1:
                case 0:
                    natTraversal = NatTraversalMethod.None;
                    break;
                case 1:
                    natTraversal = NatTraversalMethod.STUN;
                    break;
                case 2:
                    natTraversal = NatTraversalMethod.TURN;
                    break;
            }
            InitializeSoftPhone();
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            IsRegRequired = !IsRegRequired;
        }

        private void HangUp_Click(object sender, EventArgs e)
        {
            if (call != null)
            {
                if (inComingCall && call.CallState == CallState.Ringing)
                    call.Reject();
                else
                    call.HangUp();
                inComingCall = false;
                call = null;
            }
            DialedNumber.Text = string.Empty;
        }

    }
}
